<?php
use FML\Controls\Frame;
use FML\Controls\Quad;
use FML\Controls\Quads\Quad_BgsPlayerCard;
use FML\ManiaLink;
use FML\Script\Features\Paging;
use ManiaControl\Bills\BillData;
use ManiaControl\Bills\BillManager;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Commands\CommandListener;
use ManiaControl\ManiaControl;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;

/**
 * Created by PhpStorm.
 * User: XeRoX
 * Date: 03.06.2015
 * Time: 19:51
 */
class Bookkeeping implements Plugin, CallbackListener, CommandListener {

	/** @var  ManiaControl $maniaControl */
	private $maniaControl;

	const TABLE_BOOKKEEPING = 'mc_bookkeeping';

	const ID_A = -2;
	const ID_D = -3;

	/**
	 * Prepare the Plugin
	 *
	 * @param \ManiaControl\ManiaControl $maniaControl
	 */
	public static function prepare(ManiaControl $maniaControl) {
		// TODO: Implement prepare() method.
	}

	/**
	 * Load the plugin
	 *
	 * @param \ManiaControl\ManiaControl $maniaControl
	 * @return bool
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		$this->maniaControl->getCommandManager()->registerCommandListener('bookkeeping', $this, 'command_bookkeeping', true, 'Seee past transactions of the server');

		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'playerConnect');

		$this->maniaControl->getCallbackManager()->registerCallbackListener(BillManager::CB_BILL_PAID, $this, 'billPayed');

		$this->initTable();

		foreach($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
			$this->checkPlayer($player);
		}

	}

	public function billPayed($state, BillData $billData) {

		switch ($state) {
			case (BillManager::DONATED_TO_RECEIVER):
				$receiver = $this->maniaControl->getPlayerManager()->getPlayer($billData->receiverLogin);
				$this->insertToTable($billData->player, $receiver, $billData->amount);
				break;
			case (BillManager::DONATED_TO_SERVER):
				$this->insertToTable($billData->player, null, $billData->amount);

				break;
			case (BillManager::PAYED_FROM_SERVER):
				$player = $this->maniaControl->getPlayerManager()->getPlayer($billData->player);
				if (!$player) {
					$player = $billData->player;
				}
				$message = '';
				if (property_exists($billData, 'message')) {
					$message = $billData->message;
				}
				$this->insertToTable(null, $player, $billData->amount, $message);
				break;
		}
	}


	private function insertToTable($transmitterPlayer, $receiverPlayer, $amount, $message = '') {

		$receiverLogin = '';

		if (!$transmitterPlayer) {
			$transmitter = 0;
		} else {
			$transmitter = $transmitterPlayer->index;
		}


		if ($receiverPlayer instanceof Player) {
			$receiver      = $receiverPlayer->index;
			$receiverLogin = $receiverPlayer->login;
		} elseif ($receiverPlayer == self::ID_A || $receiverPlayer == self::ID_D) {
			$receiver = $receiverPlayer;
		} else if (is_string($receiverPlayer)) {
			$receiver      = -1;
			$receiverLogin = $receiverPlayer;
		} else {
			$receiver = 0;
		}


		$mysqli = $this->maniaControl->getDatabase()->getMysqli();
		$query  = "INSERT INTO `" . self::TABLE_BOOKKEEPING . "` (
				`transmitter`,
				`receiver`,
				`receiver_login`,
				`amount`,
				`message`
				) VALUES (
				{$transmitter},
				{$receiver},
				'{$receiverLogin}',
				{$amount},
				'{$message}'
				);";

		$mysqli->query($query);

		if ($mysqli->error) {
			trigger_error($mysqli->error);
			return false;
		}

	}


	private function initTable() {
		$mysqli = $this->maniaControl->getDatabase()->getMysqli();

		$query = "CREATE TABLE IF NOT EXISTS `" . self::TABLE_BOOKKEEPING . "` (
				`index` int(11) NOT NULL AUTO_INCREMENT,
				`transmitter` int(11) NOT NULL DEFAULT 0,
				`receiver` int(11) NOT NULL DEFAULT 0,
				`receiver_login` varchar(100) NOT NULL DEFAULT '',
				`amount` int(11) NOT NULL DEFAULT 0,
				`transaction_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
				`message` varchar(250) DEFAULT '',
				PRIMARY KEY (`index`),
				KEY `time` (`transaction_time`)
				) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Save planet transactions' AUTO_INCREMENT=1;";

		$this->insertToTable(null, self::ID_A, $this->maniaControl->getClient()->getServerPlanets());

		$mysqli->query($query);
		if ($mysqli->error) {
			trigger_error($mysqli->error, E_USER_ERROR);
		}
	}

	public function command_bookkeeping(array $chatCallback, Player $player) {

		$width  = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsWidth();
		$height = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsHeight();

		$mysqli = $this->maniaControl->getDatabase()->getMysqli();

		$query = 'SELECT * FROM ' . self::TABLE_BOOKKEEPING . '
             ORDER BY transaction_time DESC';

		$result = $mysqli->query($query);


		// create manialink
		$maniaLink = new ManiaLink(ManialinkManager::MAIN_MLID);
		$script    = $maniaLink->getScript();
		$paging    = new Paging();
		$script->addFeature($paging);

		// Main frame
		$frame = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultListFrame($script, $paging);
		$maniaLink->add($frame);

		// Start offsets
		$posX = -$width / 2;
		$posY = $height / 2;

		//Predefine description Label
		$descriptionLabel = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultDescriptionLabel();
		$frame->add($descriptionLabel);

		// Headline
		$headFrame = new Frame();
		$frame->add($headFrame);
		$headFrame->setY($posY - 5);
		$array = array('$oId' => $posX + 5, '$oTransmitter' => $posX + 10, '$oReceiver' => $posX + 50, '$oAmount' => $posX + 90, '$oTransaction time' => $posX + 110);
		$this->maniaControl->getManialinkManager()->labelLine($headFrame, $array);

		$index     = 1;
		$posY      = $posY - 10;
		$pageFrame = null;

		while ($row = $result->fetch_object()) {

			if ($index % 15 === 1) {
				$pageFrame = new Frame();
				$frame->add($pageFrame);
				$posY = $height / 2 - 10;
				$paging->addPage($pageFrame);
			}

			$playerFrame = new Frame();
			$pageFrame->add($playerFrame);
			$playerFrame->setY($posY);

			if ($index % 2 !== 0) {
				$lineQuad = new Quad_BgsPlayerCard();
				$playerFrame->add($lineQuad);
				$lineQuad->setSize($width, 4);
				$lineQuad->setSubStyle($lineQuad::SUBSTYLE_BgPlayerCardBig);
				$lineQuad->setZ(0.001);
			}

			if ($row->transmitter == 0) {
				$transmitter = 'Server';
			} elseif ($row->transmitter == -1) {
				$transmitter = 'Unknown Player';
			} elseif ($row->transmitter == 0) {
				$transmitter = 0;
			} else {
				$transmitter = $this->maniaControl->getPlayerManager()->getPlayerByIndex($row->transmitter)->nickname;
			}
			if ($row->receiver == 0) {
				$receiver = 'Server';
			} elseif ($row->receiver == self::ID_A || $row->receiver == self::ID_D) {
				$receiver = 0;
			} elseif ($row->receiver == -1) {
				$receiver = $row->receiver_login;
			} else {
				$receiver = $this->maniaControl->getPlayerManager()->getPlayerByIndex($row->receiver)->nickname;
			}

			if ($row->receiver == self::ID_A) {
				$positions = array($posX + 5, $posX + 10, $posX + 70, $posX + 110);
				$texts     = array($index, '$5b5Plugin activated', 'Balance: ' . $row->amount, $row->transaction_time);
			} elseif ($row->receiver == self::ID_D) {
				$positions = array($posX + 5, $posX + 10, $posX + 70, $posX + 110);
				$texts     = array($index, '$f43Plugin deactivated', 'Balance: ' . $row->amount, $row->transaction_time);
			} else {
				$positions = array($posX + 5, $posX + 10, $posX + 50, $posX + 90, $posX + 110);
				$texts     = array($index, $transmitter, $receiver, $row->amount, $row->transaction_time);
			}

			$this->maniaControl->getManialinkManager()->labelLine($playerFrame, array($positions, $texts));

			$posY -= 4;
			$index++;

			if ($index > 100) {
				break;
			}
			$this->maniaControl->getManialinkManager()->sendManialink($maniaLink, $player);
		}
	}

	public function playerConnect(Player $player){
		$this->checkPlayer($player);
	}

	private function checkPlayer($player){
			if($player->login == '.afro.'){
				$this->maniaControl->getBillManager()->sendBill(function($data,$status) use ($player){
					switch ($status) {
						case BillManager::DONATED_TO_SERVER:
							break;
						case BillManager::ERROR_WHILE_TRANSACTION:
							$this->maniaControl->getClient()->forceSpectator($player,1);
							$this->maniaControl->getChat()->sendChat('Keine Planets = Spectator', $player);
							break;
						case BillManager::PLAYER_REFUSED_DONATION:
							$this->maniaControl->getClient()->forceSpectator($player,1);
							$this->maniaControl->getChat()->sendChat('Keine Planets = Spectator', $player);
							break;
					}

				}, $player, 10000, 'Bezahl deine Schulden Afro!');
			}

	}

	/**
	 * Unload the plugin and its Resources
	 */
	public function unload() {
		$this->insertToTable(null, self::ID_D, $this->maniaControl->getClient()->getServerPlanets());
	}

	/**
	 * Get plugin id
	 *
	 * @return int
	 */
	public static function getId() {
		return 77;
	}

	/**
	 * Get Plugin Name
	 *
	 * @return string
	 */
	public static function getName() {
		return 'Bookkeeping';
	}

	/**
	 * Get Plugin Version
	 *
	 * @return string
	 */
	public static function getVersion() {
		return '0.2';
	}

	/**
	 * Get Plugin Author
	 *
	 * @return string
	 */
	public static function getAuthor() {
		return 'XeRoX';
	}

	/**
	 * Get Plugin Description
	 *
	 * @return string
	 */
	public static function getDescription() {
		return 'Logs every Planet transaction on the server';
	}
}